/*
 * Copyright (C) 2012 Tokyo Institute of Technology 
 *
 * 
 * This file is part of MEGADOCK.
 * MEGADOCK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MEGADOCK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MEGADOCK.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


//============================================================================//
//
//  Software Name : MEGADOCK
//
//  Class Name : FFTProcess
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef FFTProcess_h
#define FFTProcess_h 1

#include <algorithm>
#include <functional>

#include "cpu_time.h"
#include "parallel.h"
#include "parameter.h"
#include "protein.h"
#include "fftw3.h"

using namespace std;

typedef struct {
  float score;
  int	index[4];
} SortScore;

class FFTProcess
{
 private:
  FFTProcess(FFTProcess &c) {}
  const FFTProcess & operator=(const FFTProcess &c);
  fftw_complex			*_FFTWin;
  fftw_complex			*_FFTWout;
  CPUTime			*_cputime;
  Parallel			*_parallel;
  Parameter			*_parameter;
  vector<SortScore>		_Top;
  vector< vector<SortScore> >	_Select;
  int				_Num_fft;
  float				*_FFT_rec_r;
  float				*_FFT_rec_i;
  float				*_FFT_lig_r;
  float				*_FFT_lig_i;
  int				*_Rotation;
 protected:
  virtual void		alloc_fft();
  virtual void			fft3d(const float &theta,int myid2);
 public:
  FFTProcess(CPUTime *pcputime,Parallel *pparallel,Parameter *pparameter)
    : _cputime(pcputime),_parallel(pparallel),_parameter(pparameter) {
#ifdef DEBUG
    cout << "Constructing FFTProcess.\n";
#endif
  }
  virtual ~FFTProcess() {
#ifdef DEBUG
    cout << "Destructing FFTProcess.\n";
#endif
    delete [] _Rotation;
    delete [] _FFT_rec_r;
    delete [] _FFT_rec_i;
    delete [] _FFT_lig_r;
    delete [] _FFT_lig_i;

    fftw_free(_FFTWin); 
    fftw_free(_FFTWout); 
    fftw_cleanup();

    vector< vector<SortScore> > tmp1;
    vector<SortScore> tmp2;
    _Select.swap(tmp1);
    _Top.swap(tmp2);
  }
  virtual void		alloc_array(const int &num_fft);
  virtual void		receptor_fft(float *grid_r,float *grid_i);
  virtual void		ligand_fft(float *grid_r,float *grid_i,
				   int myid2);
  virtual void		fft_score(int myid2);
  virtual void		score_sort(int myid2);
  virtual void		top_score_clean();
  virtual int		num_fft() { return _Num_fft; }
  virtual void		num_fft(const int &i) { _Num_fft = i; }
  virtual void		rotation_index(int i,const int &j) { _Rotation[i] = j; }
  virtual float		top_score(const int &j) {
    return _Top[j].score;
  }
  virtual int		top_index(const int &j,int k) {
    return _Top[j].index[k];
  }
  virtual void		sort_index(float *fwork,int *iwork);
};

#endif
