/*
 * Copyright (C) 2012 Tokyo Institute of Technology 
 *
 * 
 * This file is part of MEGADOCK.
 * MEGADOCK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MEGADOCK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MEGADOCK.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


//============================================================================//
//
//  Software Name : MEGADOCK
//
//  Class Name : (main)
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "cpu_time.h"
#include "control.h"

#define VERSION "K3.0"
#define LASTUPDATED "2012/9/12"

//============================================================================//
#ifdef USE_MPI
int application(int argc,char *argv[])
#else
int main( int argc, char *argv[])
#endif
//============================================================================//
{
  Parallel	*_parallel;
  CPUTime	*_cputime;
  Control	*_control;

  struct timeval et1, et2;
  int 		nproc2;

  gettimeofday(&et1,NULL);

  cout << " MEGADOCK ver. "<< VERSION << endl;
  cout << "      megadock@bi.cs.titech.ac.jp   lastupdated: " << LASTUPDATED << endl;
  cout << endl;

#ifdef _OPENMP
#pragma omp parallel
  {
#pragma omp single
   {
    nproc2 = omp_get_num_threads();
    cout << "#Using OpenMP parallelization: " << nproc2 << " threads." << endl;
   }
  }
#else
  nproc2 = 1;
#endif

  _cputime = new CPUTime();
  _cputime->initialize();

  _parallel = new Parallel(nproc2);

  _control = new Control(_cputime,_parallel);
  _control->initialize(argc,argv);
  _control->execute();

  delete _control;
  delete _parallel;

  _cputime->output();

  delete _cputime;

  gettimeofday(&et2,NULL);

  cout << "\nElapsed time  = " << et2.tv_sec-et1.tv_sec
    + (float)((et2.tv_usec-et1.tv_usec)*1e-6) << " sec.\n";

  return 0;
}
