/*
 * Copyright (C) 2012 Tokyo Institute of Technology 
 *
 * 
 * This file is part of MEGADOCK.
 * MEGADOCK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MEGADOCK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MEGADOCK.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


//============================================================================//
//
//  Software Name : MEGADOCK
//
//  Class Name : Parameter
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#ifndef Parameter_h
#define Parameter_h 1

#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <map>
#include <cassert>
#include <unistd.h>

#include "constant.h"
#include "parallel.h"

using namespace std;

class Parameter
{
 private:
  friend class			Control;
  friend class			Docking;
  friend class			FFTProcess;
  friend class			Protein;
  friend class			HeaderDB;
  Parameter(Parameter &c){}
  const Parameter & operator=(const Parameter &c);
  Parallel			*_parallel;
  string			_RecPDB_file;
  string			_LigPDB_file;
  string			_RLOut_file;
  int				_IO_flag[3];
  int				_Num_grid;
  int				_Num_fft;
  int				_Num_fft_flag;
  int				_Num_output;
  int				_Num_output_flag;
  int				_Score_func;
  int				_Num_sort;
  float				_Elec_ratio;
  float				_Grid_size;
  float				_Delta_angle;
  float				_Grid_space_rec;
  float				_Grid_space_lig;
  float				*_Zangle;
  int				_NZangle;
  map<string,float>		_Charmmr;
  map<string,float>		_Charmmc;
 protected:
  virtual void			usage();
  virtual void			default_param();
  virtual void			pdb_step();
  virtual void			parameter_set();
  virtual void			charmm_radius();
  virtual void			charmm_charge();
  virtual void			dangle15();
  virtual void			dangle6();
 public:
  Parameter(Parallel *pparallel) : _parallel(pparallel) {
#ifdef DEBUG
    cout << "Constructing Parameter.\n";
#endif
  }
  virtual			~Parameter() {
#ifdef DEBUG
    cout << "Destructing Parameter.\n";
#endif
    delete [] _Zangle;
  }
  virtual void			initialize(int argc,char *argv[]);
  virtual float			atom_radius(const string &atype);
  virtual float			atom_charge(const string &atype);
};

#endif
